require_relative "./element"


module ObjectView 

  class TextArea < ObjectView::Element 
    def initialize(value = "")
      super()
      @tag = "textarea"
      
    end
    
    def rows= row_count
      self.attr('rows', row_count.to_s)
    end

    def columns= count
      self.attr('cols', row_count.to_s)
    end
    
    def value= text
      self.children.clear if (self.children)
      self.add text
    end
  end

  class Input < ObjectView::Element 
    
    def initialize
      super()
      @tag = "input"
    end
    
    def value= text
      self.attr('value', text)
    end
    
    def value
      return self.attr('value')
    end
    
    def type= text
      self.attr('type', text)
    end
    
    def type
      return self.attr('type')
    end
    
    def name= text
      self.attr('name', text)
    end
    
    def name
      self.attributes['name']
    end
    
    def read_only
      self.attr('readonly')
    end
    
    def read_only= value
      self.attr('readonly', value)
    end
  end  
  

  class Radio < ObjectView::Input
    def initialize
      super
      self.type = 'radio'
    end
  end
  
  class Checkbox < ObjectView::Input
    def initialize
      super
      self.type = 'checkbox'
    end
  end
  
  class TextInput < ObjectView::Input
    def place_holder= text
      self.attr('value', text)
    end
  end

  class PasswordInput < ObjectView::Input
    def initialize
      super
      self.type = 'password'
    end

    def place_holder= text
      self.attr('value', text)
    end
  end
  
  class CheckInput < Input
    def initialize
      super
      self.type = 'checkbox'
    end
  end

  class SubmitInput < Input
    def initialize
      super
      self.type = 'submit'
    end
  end
  

  class Option < ObjectView::Element
    def initialize(name, value)
      super()
      @tag = 'option'
      self.attr('value', value)
      self.add name
    end    
  end

  class HiddenInput < Input
    def initialize
      super
      self.type = 'hidden'
    end
  end

  
  class Combobox < ObjectView::Element
    def initialize
      super
      @tag = 'select'
    end
    
    def option(name, value)
      option = self.add Option.new(name, value)
      return option
    end
    
    def name= text
      self.attr('name', text)
    end
    
    def name
      self.attributes['name']
    end
    
    def select(select_option)
      self.children.each do |option|
        if (selection_option == option)
          option.attr('selected', 'selected')
        else
          if (option.attributes['selected'] != nil)
            option.attributes.delete('selected')
          end
        end
      end
    end
    
  end
  
  class Form < ObjectView::Element
    def initialize
      super
      @tag = 'form'
      self.attr('method', "post")
    end
    
    def action= text
      self.attr('action', text)
    end
    
    def accept_charset=(value)
      self.attr('accept-charset', value)
    end
    
    def action=(value)
      self.attr('action', value)
    end

    def method=(value)
      self.attr('method', value)
    end
    
  end
  
  class Button < ObjectView::Element
    def initialize(text = nil, name = nil)
      super(text)
      @tag = 'button'
      self.attr('type', 'submit')
      if (name != nil)
        self.attr('name', name)
      end
    end
    
    def label= text
      self.add text
    end
  end
  
  class Submit < ObjectView::Input
    def initialize(text = nil, name = nil)
      super()
      @tag = 'input'
      if (text != nil)
        self.attr('value', text)
      end
      self.attr('type', 'submit')
      if (name != nil)
        self.attr('name', name)
      end
    end
    
    
    def label= text
      self.add text
    end
  end
  
  
  class Label < ObjectView::Element
    def initialize(content = nil)
      super(content)
      @tag = 'label'
    end
    
    def for=(value)
      attr('for', value)
    end
  end
  
  class FieldSet < ObjectView::Element
    def initialize(content = nil)
      super(content)
      @tag = 'fieldset'
    end
  end
  
end
