require_relative "./javascript"
module ObjectView 

class HighChartLoadJavascriptAjax < ObjectView::Javascript

  def initialize(chart_data_list, prepend_script)
    super()
    
    script = ""
    if (prepend_script != nil)
      script += prepend_script
    end
    
    script += chart_class_script()
    
    
    script += "
        _chartHash = new Object();
        _chartList = new Array();
     "
    chart_data_list.each do |chart_data|
      params_line = params_for_chart(chart_data)
      refresh_seconds = chart_data.ajax_refresh_seconds 
      if (refresh_seconds == nil)
        refresh_seconds = "null"
      end
      
      #   'apple=green&extent=' + _extent + ''
      #   'extent=' + _extent + '&' + 'apple=green' + '&' + 'banana=yellow'
      script += "    
          #{params_line}
       var chartDraw = new HighChartDraw('#{chart_data.id}', '#{chart_data.ajax_url}', params, #{refresh_seconds});
       _chartHash['#{chart_data.id}'] = chartDraw;
       _chartList.push(chartDraw);
       console.log('_chartList.length: ' + _chartList.length);
         "         
    end
    script += "
        function periodicCheckForChartDraw() {
          var chartCount = _chartList.length;
          //console.log('periodicCheckForChartDraw()  count: ' + chartCount);
          for (i = 0; i < chartCount; i++) {
            chart = _chartList[i];
            chart.periodicCheck();
          }
        }
      	setInterval(periodicCheckForChartDraw, 500);
    "
    
    self.add script
  end
  
  def chart_class_script
    script = 
    "
    
    function prepareChartData(chartData) {
      console.log('Data type: ' + typeof(chartData));
      prepareObjectAndChildren(chartData);
    }
    
    
    
    function prepareObjectAndChildren(object) {
      if (typeof(object) == 'object') {
        for (var key in object) {
            if (object.hasOwnProperty(key)) {
                //console.log('key is: ' + key + ', value is: ' + object[key]);
                child = object[key];
                if (typeof(child) == \"object\" ) {
                  prepareObjectAndChildren(child);
                } else if (typeof(child) == 'string') {
                  //console.log('STRING key is: ' + key + ', value is: ' + object[key]);
                  if (child.indexOf('/@') == 0) {
                    var functionCode = child.replace('/@', '');
                    functionCode = functionCode.replace('@/', '');
                    object[key] = function() {
                          return eval(functionCode);
                      }
                    //console.log('STRING (functionCode): ' + key + ', value is: ' + functionCode);
                    //console.log('STRING was function ' + key + ', value is: ' + object[key]);
                  }
                }
            }
        }
      }
    }
    //response = response.replace('\"@@', '');
    //response = response.replace('@@\"', '');
    
    
    function HighChartDraw(chartId, ajaxUrl, ajaxParams, refreshSeconds) {
      console.log('Created HighChartDraw  (' + chartId + ')');
    	this.nextTimeToDraw = 0;
      this.currentlyDrawing = false;
    	this.ajaxUrl = ajaxUrl;
    	this.ajaxParams = ajaxParams;
    	this.refreshSeconds = refreshSeconds;
    	this.chartId = chartId;
      console.log('Initializing chart: ' + this.chartId);
    	this.chartRequestMade = false;
    	this.clearChart();
    }
   
    HighChartDraw.prototype.buildParamString = function() {
      var paramString = '';
      for (var paramName in this.ajaxParams) {
          if (this.ajaxParams.hasOwnProperty(paramName)) {
            paramValue = this.ajaxParams[paramName];
            if (paramString.length > 0) {
              paramString = paramString + '&';
            }
            paramString += paramName + '=';
            var indexOfVal = paramValue.indexOf('VAR|');
            if (indexOfVal == 0) {
              var variable = paramValue.substring(4, paramValue.length);
              console.log('Variable for eval: ' + variable);
              paramString += encodeURIComponent(eval(variable));
              //paramString += eval(variable);
            } else {
              paramString += encodeURIComponent(paramValue);
              //paramString += paramValue;
            }
          }
      }
      console.log('Post params: ' + paramString);
      return paramString;      
    }


    HighChartDraw.prototype.draw = function() {
        try {
            this.currentlyDrawing = true;
            console.log('Drawing chart (' + this.chartId + ')');
            var chartElementId = 'chart_1';
    	    	var chartId = this.chartId;
            $.ajax({
                type: 'GET',
                url: this.ajaxUrl,
                data: this.buildParamString(),
                dataType: 'text',
                success: function(response) {
                  var chartData = jQuery.parseJSON(response);
                  prepareChartData(chartData);
                  $('#' + chartId).highcharts(chartData);
                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                  $('#' + chartId).html('<h3>Error getting chart data - QA server may be down.</h3>');
                }
            });
            console.log('finished drawing chart.  Queuing another drawHighChart request.');
            if (this.refreshSeconds != null) {
              this.makeChartDrawRequest(1000 * this.refreshSeconds);
            }
            this.currentlyDrawing = false;
        } catch(error) {
           console.log('Exception drawing chart: ' + error);
           this.currentlyDrawing = false;
           this.makeChartDrawRequest(1000);
           throw error;
        }
        this.currentlyDrawing = false;
    }

    HighChartDraw.prototype.clearChart = function() {
    	$('#' + this.chartId).html('<h2 style=\"color: #aaaaaa\">Building Chart...</h2>'); 
    }

    HighChartDraw.prototype.makeChartDrawRequest = function(millisFromNow) {
        var nextTime = jQuery.now() + millisFromNow;
        if ((this.nextTimeToDraw == null) || (nextTime < this.nextTimeToDraw)) {
          this.nextTimeToDraw = nextTime;
        }
        console.log('Request to drawHighChart (' + this.chartId + ') queued for: ' + this.nextTimeToDraw);
    }

    HighChartDraw.prototype.periodicCheck = function() {
      //console.log('periodicCheck for ' + this.chartId);
    	var now = jQuery.now();
    	if (this.nextTimeToDraw != null) {
    	  if (now > this.nextTimeToDraw) {
    	    this.doingPeriodicCheck = false;
    	    if (this.currentlyDrawing == false) {
    	      this.nextTimeToDraw = null;
    	      this.draw();
    	    } else {
            //console.log('periodicCheck for ' + this.chartId + '  currently drawing, so ignoring request');
    	    }
    	  } else {
          //console.log('periodicCheck for ' + this.chartId + '  next draw time not reached');
        }
    	} else {
          //console.log('periodicCheck for ' + this.chartId + '  nextTimeToDraw is null');
      }
    	this.doingPeriodicCheck = false;
    }
    "
    return script
  end
  
  

  def draw_chart_function(ajax_refresh_seconds, chart_data_list)
    script = "
      function drawHighChart() {
          console.log('drawHighChart called');
      "
      script += "
          try {
              _currentlyDrawing = true;
              console.log('Drawing charts.');
              //alert('drawHighChart()');
              "
    script +=  "     #{chart_data_load_script(chart_data_list)}\n" 
    
    if (ajax_refresh_seconds != nil)   
      script += "     
               console.log('finished drawing chart.  Queuing another drawHighChart request.');
               makeChartDrawRequest(#{ajax_refresh_seconds * 1000});\n
               _currentlyDrawing = false;
      "
    end
      
    script += "    
          }  catch(error) {
             console.log('Exception drawing chart: ' + error);
             _currentlyDrawing = false;
             makeChartDrawRequest(1000);
          }
          "
          
    script +=   "
          _currentlyDrawing = false;
        }
     "
     return script
  end


  def clear_script_for_each_chart(chart_data_list) 
    script = ""
   	chart_data_list.each do |chart_data|  
      chart_data.id
      script += "  $('##{chart_data.id}').html('<h3 style=\"color: #aaaaaa\">Building Chart</h3>');\n"
    end
    return script
  end  

  def periodic_check_function(chart_data_list)
    script = "
    function periodicCheck(chartId) {
        var now = jQuery.now();
        if (_nextTimeToDraw[chartId] != null) {
          if (now > _nextTimeToDraw[chartId]) {
            _doingPeriodicCheck = false;
            if (_currentlyDrawing == false) {
              _nextTimeToDraw[chartId] = null;
              drawHighChart(chartId);
            } else {
            }
          }
        }
        _doingPeriodicCheck = false;
      }
    }
    "
    
    script += 
    "
    function periodicCheckAll() {
      if (!_doingPeriodicCheck) {
        _doingPeriodicCheck = true;
        "
        
    chart_data_list.each do |chart_data|
      script += "   periodicCheck('#{chart_data.id}'); "
    end    
        
    script +=    
        "
      }
    }
    "
  end

  def params_for_chart(chart_data)
    params_str = "var params = new Object();\n"

    chart_data.ajax_params.each do |name, value|
        params_str += "     params['#{name}'] = '#{value}';\n"
    end
    
    return params_str
  end

  # def params_for_chart(chart_data)
  #   params_str = "var params = '' + "
  # 
  #   chart_data.ajax_params.each do |name, value|
  #   #  params_string += "#{name}=#{value}&"
  #     if (value.to_s.start_with? "VAR|")
  #       params_str += "'#{name}=' + #{value[4..(value.length - 1)]}"
  #     else
  #       encoded_value = URI.encode_www_form([[name, value]])
  #       params_str += "'#{encoded_value}'"
  #       #params_str += "'#{name}=#{encoded_value}'"
  #     end
  #     params_str += "+'&'+"
  #   end
  #   
  #   params_str = params_str[0..params_str.length - 6]
  #   params_str += ';'
  #   # if (params_string.length > 0)
  #   #   params_string = params_string[0..params_string.length - 2]
  #   # end
  #   # params_string = URI.encode_www_form(params)
  #   return params_str
  # end
  
  def chart_data_load_script(chart_data_list)
    script = ""
   	chart_data_list.each do |chart_data|  
      
    
      script += 
      "
                var chartElementId = '#{chart_data.id}';
                $.ajax({
                  type: 'GET',
                  url: '#{chart_data.ajax_url}',
                  data: params,
                  dataType: 'text',
                  success: function(response) {
                    //alert('Response: ' + response);
                    var chartData = jQuery.parseJSON(response);
                    //alert('Response: ' + response);
                    $('#' + chartElementId).highcharts(chartData);"
                    
      
      if (chart_data.ajax_refresh_seconds != nil)                    
        script += "              
                    makeChartDrawRequest(#{chart_data.ajax_refresh_seconds * 1000});
            "        
      end
                    
      script += "
                  },
                  error: function(XMLHttpRequest, textStatus, errorThrown) {
                    $('#' + chartElementId).html('<h3>Error getting chart data - QA server may be down.</h3>');
                  }
                });
      "
     end
     return script    
  end  

end


end # Module